@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
+from sys import maxsize
 from typing import Any, Dict, List, NamedTuple, NewType, Optional, Union
 
 from base58 import b58decode, b58encode
@@ -57,7 +58,9 @@ class NonceInformation(NamedTuple):
 
 
 @dataclass
-class _SigPubkeyPair:
+class SigPubkeyPair:
+    """Pair of signature and corresponding public key."""
+
     pubkey: PublicKey
     signature: Optional[bytes] = None
 
@@ -72,11 +75,13 @@ class Transaction:
         self,
         recent_blockhash: Optional[Blockhash] = None,
         nonce_info: Optional[NonceInformation] = None,
-        signatures: Optional[List[_SigPubkeyPair]] = None,
+        signatures: Optional[List[SigPubkeyPair]] = None,
+        fee_payer: Optional[PublicKey] = None,
     ) -> None:
         """Init transaction object."""
+        self.fee_payer = fee_payer
         self.instructions: List[TransactionInstruction] = []
-        self.signatures: List[_SigPubkeyPair] = signatures if signatures else []
+        self.signatures: List[SigPubkeyPair] = signatures if signatures else []
         self.recent_blockhash, self.nonce_info = recent_blockhash, nonce_info
 
     def __eq__(self, other: Any) -> bool:
@@ -117,6 +122,14 @@ class Transaction:
         if len(self.instructions) < 1:
             raise AttributeError("no instructions provided")
 
+        fee_payer = self.fee_payer
+        if not fee_payer and len(self.signatures) > 0 and self.signatures[0].pubkey:
+            # Use implicit fee payer
+            fee_payer = self.signatures[0].pubkey
+
+        if not fee_payer:
+            raise AttributeError("transaction feePayer required")
+
         account_metas, program_ids = [], set()
         for instr in self.instructions:
             if not instr.program_id or not instr.keys:
@@ -128,12 +141,11 @@ class Transaction:
         for pg_id in program_ids:
             account_metas.append(AccountMeta(PublicKey(pg_id), False, False))
 
-        # Prefix accountMetas with feePayer here whenever that gets implemented.
-
         # Sort. Prioritizing first by signer, then by writable and converting from set to list.
         account_metas.sort(key=lambda account: (not account.is_signer, not account.is_writable))
 
         # Cull duplicate accounts
+        fee_payer_idx = maxsize
         seen: Dict[str, int] = dict()
         uniq_metas: List[AccountMeta] = []
         for sig in self.signatures:
@@ -143,6 +155,8 @@ class Transaction:
             else:
                 uniq_metas.append(AccountMeta(sig.pubkey, True, True))
                 seen[pubkey] = len(uniq_metas) - 1
+                if sig.pubkey == fee_payer:
+                    fee_payer_idx = min(fee_payer_idx, seen[pubkey])
 
         for a_m in account_metas:
             pubkey = str(a_m.pubkey)
@@ -152,23 +166,32 @@ class Transaction:
             else:
                 uniq_metas.append(a_m)
                 seen[pubkey] = len(uniq_metas) - 1
+                if a_m.pubkey == fee_payer:
+                    fee_payer_idx = min(fee_payer_idx, seen[pubkey])
+
+        # Move fee payer to the front
+        if fee_payer_idx == maxsize:
+            uniq_metas = [AccountMeta(fee_payer, True, True)] + uniq_metas
+        else:
+            uniq_metas = (
+                [uniq_metas[fee_payer_idx]] + uniq_metas[:fee_payer_idx] + uniq_metas[fee_payer_idx + 1 :]  # noqa: E203
+            )
 
         # Split out signing from nonsigning keys and count readonlys
         signed_keys: List[str] = []
         unsigned_keys: List[str] = []
-        num_readonly_signed_accounts = num_readonly_unsigned_accounts = 0
+        num_required_signatures = num_readonly_signed_accounts = num_readonly_unsigned_accounts = 0
         for a_m in uniq_metas:
             if a_m.is_signer:
-                # Promote the first signer to writable as it is the fee payer
-                if len(signed_keys) != 0 and not a_m.is_writable:
-                    num_readonly_signed_accounts += 1
                 signed_keys.append(str(a_m.pubkey))
+                num_required_signatures += 1
+                num_readonly_signed_accounts += int(not a_m.is_writable)
             else:
                 num_readonly_unsigned_accounts += int(not a_m.is_writable)
                 unsigned_keys.append(str(a_m.pubkey))
         # Initialize signature array, if needed
         if not self.signatures:
-            self.signatures = [_SigPubkeyPair(pubkey=PublicKey(key), signature=None) for key in signed_keys]
+            self.signatures = [SigPubkeyPair(pubkey=PublicKey(key), signature=None) for key in signed_keys]
 
         account_keys: List[str] = signed_keys + unsigned_keys
         account_indices: Dict[str, int] = {str(key): i for i, key in enumerate(account_keys)}
@@ -184,7 +207,7 @@ class Transaction:
         return Message(
             MessageArgs(
                 header=MessageHeader(
-                    num_required_signatures=len(self.signatures),
+                    num_required_signatures=num_required_signatures,
                     num_readonly_signed_accounts=num_readonly_signed_accounts,
                     num_readonly_unsigned_accounts=num_readonly_unsigned_accounts,
                 ),
@@ -211,8 +234,8 @@ class Transaction:
         def partial_signer_pubkey(account_or_pubkey: Union[PublicKey, Account]):
             return account_or_pubkey.public_key() if isinstance(account_or_pubkey, Account) else account_or_pubkey
 
-        signatures: List[_SigPubkeyPair] = [
-            _SigPubkeyPair(pubkey=partial_signer_pubkey(partial_signer)) for partial_signer in partial_signers
+        signatures: List[SigPubkeyPair] = [
+            SigPubkeyPair(pubkey=partial_signer_pubkey(partial_signer)) for partial_signer in partial_signers
         ]
         self.signatures = signatures
         sign_data = self.serialize_message()
@@ -243,7 +266,7 @@ class Transaction:
         if len(signature) != SIG_LENGTH:
             raise ValueError("signature has invalid length", signature)
         idx = next((i for i, sig_pair in enumerate(self.signatures) if sig_pair.pubkey == pubkey), None)
-        if not idx:
+        if idx is None:
             raise ValueError("unknown signer: ", str(pubkey))
         self.signatures[idx].signature = signature
 
@@ -280,12 +303,12 @@ class Transaction:
         >>> from solana.publickey import PublicKey
         >>> from solana.system_program import transfer, TransferParams
         >>> sender, reciever = Account(1), PublicKey(2)
-        >>> transfer_tx = transfer(TransferParams(from_pubkey=sender.public_key(), to_pubkey=reciever, lamports=1000))
+        >>> transfer_tx = Transaction().add(transfer(TransferParams(from_pubkey=sender.public_key(), to_pubkey=reciever, lamports=1000)))
         >>> transfer_tx.recent_blockhash = Blockhash(str(PublicKey(3)))
         >>> transfer_tx.sign(sender)
         >>> transfer_tx.serialize().hex()
         '019d53be8af3a7c30f86c1092d2c3ea61d270c0cfa275a23ba504674c8fbbb724827b23b42dc8e08019e23120f1b6f40f9799355ce54185b4415be37ca2cee6e0e010001034cb5abf6ad79fbf5abbccafcc269d85cd2651ed4b885b5869f241aedf0a5ba2900000000000000000000000000000000000000000000000000000000000000020000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000301020200010c02000000e803000000000000'
-        """  # pylint: disable=line-too-long
+        """  # noqa: E501 pylint: disable=line-too-long
         if not self.signatures:
             raise AttributeError("transaction has not been signed")
 
@@ -304,11 +327,13 @@ class Transaction:
         wire_transaction.extend(signature_count)
         # Encode signatures
         for sig_pair in self.signatures:
-            if not sig_pair.signature:
-                continue
-            if len(sig_pair.signature) != SIG_LENGTH:
+            if sig_pair.signature and len(sig_pair.signature) != SIG_LENGTH:
                 raise RuntimeError("signature has invalid length", sig_pair.signature)
-            wire_transaction.extend(sig_pair.signature)
+
+            if not sig_pair.signature:
+                wire_transaction.extend(bytearray(SIG_LENGTH))
+            else:
+                wire_transaction.extend(sig_pair.signature)
         # Encode signed data
         wire_transaction.extend(signed_data)
 
@@ -368,7 +393,7 @@ class Transaction:
 
         for idx, sig in enumerate(signatures):
             signature = None if sig == b58encode(Transaction.__DEFAULT_SIG) else b58decode(sig)
-            transaction.signatures.append(_SigPubkeyPair(pubkey=message.account_keys[idx], signature=signature))
+            transaction.signatures.append(SigPubkeyPair(pubkey=message.account_keys[idx], signature=signature))
 
         for instr in message.instructions:
             account_metas: List[AccountMeta] = []
@@ -384,13 +409,3 @@ class Transaction:
             )
 
         return transaction
-
-
-def verify_instruction_keys(instruction: TransactionInstruction, expected: int) -> None:
-    """Verify length of AccountMeta list of a transaction instruction is at least the expected length.
-
-    :param instruction: A TransactionInstruction object.
-    :param expected: Expected length.
-    """
-    if len(instruction.keys) < expected:
-        raise ValueError(f"invalid instruction: found {len(instruction.keys)} keys, expected at least {expected}")
